var DocUtils,
  __slice = [].slice;

DocUtils = {};

DocUtils.getPathConfig = function() {
  if (DocUtils.pathConfig == null) {
    return "";
  }
  if (DocUtils.env === 'node') {
    return DocUtils.pathConfig.node;
  }
  return DocUtils.pathConfig.browser;
};

DocUtils.escapeRegExp = function(str) {
  return str.replace(/[\-\[\]\/\{\}\(\)\*\+\?\.\\\^\$\|]/g, "\\$&");
};

DocUtils.charMap = {
  '&': "&amp;",
  "'": "&apos;",
  "<": "&lt;",
  ">": "&gt;"
};

DocUtils.wordToUtf8 = function(string) {
  var endChar, startChar, _ref;
  _ref = DocUtils.charMap;
  for (endChar in _ref) {
    startChar = _ref[endChar];
    string = string.replace(new RegExp(DocUtils.escapeRegExp(startChar), "g"), endChar);
  }
  return string;
};

DocUtils.utf8ToWord = function(string) {
  var endChar, startChar, _ref;
  _ref = DocUtils.charMap;
  for (startChar in _ref) {
    endChar = _ref[startChar];
    string = string.replace(new RegExp(DocUtils.escapeRegExp(startChar), "g"), endChar);
  }
  return string;
};

DocUtils.defaultParser = function(tag) {
  return {
    'get': function(scope) {
      if (tag === '.') {
        return scope;
      } else {
        return scope[tag];
      }
    }
  };
};

DocUtils.tags = {
  start: '{',
  end: '}'
};

DocUtils.clone = function(obj) {
  var flags, key, newInstance;
  if ((obj == null) || typeof obj !== 'object') {
    return obj;
  }
  if (obj instanceof Date) {
    return new Date(obj.getTime());
  }
  if (obj instanceof RegExp) {
    flags = '';
    if (obj.global != null) {
      flags += 'g';
    }
    if (obj.ignoreCase != null) {
      flags += 'i';
    }
    if (obj.multiline != null) {
      flags += 'm';
    }
    if (obj.sticky != null) {
      flags += 'y';
    }
    return new RegExp(obj.source, flags);
  }
  newInstance = new obj.constructor();
  for (key in obj) {
    newInstance[key] = DocUtils.clone(obj[key]);
  }
  return newInstance;
};

DocUtils.replaceFirstFrom = function(string, search, replace, from) {
  return string.substr(0, from) + string.substr(from).replace(search, replace);
};

DocUtils.encode_utf8 = function(s) {
  return unescape(encodeURIComponent(s));
};

DocUtils.convert_spaces = function(s) {
  return s.replace(new RegExp(String.fromCharCode(160), "g"), " ");
};

DocUtils.decode_utf8 = function(s) {
  var e;
  try {
    if (s === void 0) {
      return void 0;
    }
    return decodeURIComponent(escape(DocUtils.convert_spaces(s)));
  } catch (_error) {
    e = _error;
    console.error(s);
    console.error('could not decode');
    throw new Error('end');
  }
};

DocUtils.base64encode = function(b) {
  return btoa(unescape(encodeURIComponent(b)));
};

DocUtils.preg_match_all = function(regex, content) {

  /*regex is a string, content is the content. It returns an array of all matches with their offset, for example:
  	regex=la
  	content=lolalolilala
  	returns: [{0:'la',offset:2},{0:'la',offset:8},{0:'la',offset:10}]
   */
  var matchArray, replacer;
  if (!(typeof regex === 'object')) {
    regex = new RegExp(regex, 'g');
  }
  matchArray = [];
  replacer = function() {
    var match, offset, pn, string, _i;
    match = arguments[0], pn = 4 <= arguments.length ? __slice.call(arguments, 1, _i = arguments.length - 2) : (_i = 1, []), offset = arguments[_i++], string = arguments[_i++];
    pn.unshift(match);
    pn.offset = offset;
    return matchArray.push(pn);
  };
  content.replace(regex, replacer);
  return matchArray;
};

DocUtils.sizeOfObject = function(obj) {
  var key, log, size;
  size = 0;
  log = 0;
  for (key in obj) {
    size++;
  }
  return size;
};

DocUtils.getOuterXml = function(text, start, end, xmlTag) {
  var endTag, startTag;
  endTag = text.indexOf('</' + xmlTag + '>', end);
  if (endTag === -1) {
    throw new Error("can't find endTag " + endTag);
  }
  endTag += ('</' + xmlTag + '>').length;
  startTag = Math.max(text.lastIndexOf('<' + xmlTag + '>', start), text.lastIndexOf('<' + xmlTag + ' ', start));
  if (startTag === -1) {
    throw new Error("can't find startTag");
  }
  return {
    "text": text.substr(startTag, endTag - startTag),
    startTag: startTag,
    endTag: endTag
  };
};

module.exports = DocUtils;
